@@ -19,6 +19,7 @@
 
 #include "component.h"
 #include "screenshot.h"
+#include "release.h"
 #include <QSharedData>
 #include <QStringList>
 #include <QUrl>
@@ -34,6 +35,7 @@ static bool operator<(QSize a, QSize b)
 
 class Appstream::ComponentData : public QSharedData {
     public:
+        QString m_id;
         QStringList m_categories;
         QStringList m_compulsoryForDesktops;
         QString m_description;
@@ -41,7 +43,6 @@ class Appstream::ComponentData : public QSharedData {
         QStringList m_extends;
         QStringList m_extensions;
         QString m_icon;
-        QString m_id;
         Component::Kind m_kind;
         QString m_name;
         QStringList m_packageNames;
@@ -53,6 +54,7 @@ class Appstream::ComponentData : public QSharedData {
         QList<Appstream::Screenshot> m_screenshots;
         QMultiHash<Provides::Kind, Provides> m_provides;
         QHash<Component::BundleKind, QString> m_bundles;
+        QList<Release> m_releases;
         bool operator==(const ComponentData& other) const {
             if(m_categories != other.m_categories) {
                 return false;
@@ -108,6 +110,9 @@ class Appstream::ComponentData : public QSharedData {
             if(m_bundles != other.m_bundles) {
                 return false;
             }
+            if(m_releases != other.m_releases) {
+                return false;
+            }
 
             // we don't check for m_extensions, since this is auto-generated and not a specific property of a component.
 
@@ -148,6 +153,15 @@ QString Component::id() const {
     return d->m_id;
 }
 
+QString Component::desktopId() const {
+    if (d->m_kind != Component::KindDesktop)
+        return QString();
+
+    if (d->m_id.endsWith(".desktop"))
+        return d->m_id;
+    return d->m_id + QStringLiteral(".desktop");
+}
+
 bool Component::isCompulsoryForDesktop(const QString& desktop) const {
     return d->m_compulsoryForDesktops.contains(desktop);
 }
@@ -302,13 +316,17 @@ Component::~Component() = default;
 
 typedef QHash<Component::Kind, QString> KindMap;
 Q_GLOBAL_STATIC_WITH_ARGS(KindMap, kindMap, ( {
-    { Component::KindAddon, QLatin1String("addon") },
-    { Component::KindCodec, QLatin1String("codec") },
-    { Component::KindDesktop, QLatin1String("desktop") },
-    { Component::KindFont, QLatin1String("font") },
     { Component::KindGeneric, QLatin1String("generic") },
+    { Component::KindDesktop, QLatin1String("desktop-application") },
+    { Component::KindConsoleApp, QLatin1String("console-application") },
+    { Component::KindWebApp, QLatin1String("web-application") },
+    { Component::KindAddon, QLatin1String("addon") },
+    { Component::KindFont, QLatin1String("font") },
+    { Component::KindCodec, QLatin1String("codec") },
     { Component::KindInputmethod, QLatin1String("inputmethod") },
-    { Component::KindAddon, QLatin1String("firmware") },
+    { Component::KindFirmware, QLatin1String("firmware") },
+    { Component::KindDriver, QLatin1String("driver") },
+    { Component::KindLocalization, QLatin1String("localization") },
     { Component::KindUnknown, QLatin1String("unknown") }
     }
 ));
@@ -318,27 +336,42 @@ QString Component::kindToString(Component::Kind kind) {
 }
 
 Component::Kind Component::stringToKind(const QString& kindString) {
-    if(kindString ==  QLatin1String("generic")) {
+    if(kindString ==  QLatin1String("")) {
         return KindGeneric;
     }
-    if (kindString == QLatin1String("desktop")) {
+    if(kindString ==  QLatin1String("generic"))
+        return KindGeneric;
+
+    if (kindString == QLatin1String("desktop-application"))
         return KindDesktop;
-    }
-    if (kindString == QLatin1String("font")) {
-        return KindFont;
-    }
-    if (kindString == QLatin1String("codec")) {
-        return KindCodec;
-    }
-    if (kindString==QLatin1String("inputmethod")) {
-        return KindInputmethod;
-    }
-    if (kindString == QLatin1String("addon")) {
+
+    if (kindString == QLatin1String("console-application"))
+        return KindConsoleApp;
+
+    if (kindString == QLatin1String("web-application"))
+        return KindWebApp;
+
+    if (kindString == QLatin1String("addon"))
         return KindAddon;
-    }
-    if (kindString == QLatin1String("firmware")) {
+
+    if (kindString == QLatin1String("font"))
+        return KindFont;
+
+    if (kindString == QLatin1String("codec"))
+        return KindCodec;
+
+    if (kindString==QLatin1String("inputmethod"))
+        return KindInputmethod;
+
+    if (kindString == QLatin1String("firmware"))
         return KindFirmware;
-    }
+
+    if (kindString == QLatin1String("driver"))
+        return KindDriver;
+
+    if (kindString == QLatin1String("localization"))
+        return KindLocalization;
+
     return KindUnknown;
 
 }
@@ -423,6 +456,16 @@ void Component::setProvides(const QList<Appstream::Provides>& provides) {
     }
 }
 
+QList<Appstream::Release> Appstream::Component::releases() const
+{
+    return d->m_releases;
+}
+
+void Appstream::Component::setReleases(const QList<Appstream::Release>& releases)
+{
+    d->m_releases = releases;
+}
+
 bool Component::isValid() const
 {
     return !d->m_id.isEmpty() &&
