@@ -37,6 +37,7 @@
 
 #include "as-utils.h"
 #include "as-utils-private.h"
+#include "as-pool-private.h"
 
 #define YAML_SEPARATOR "---"
 /* Compilers will optimise this to a constant */
@@ -61,12 +62,20 @@ as_get_yml_data_origin (const gchar *fname)
 	g_autoptr(GFile) file = NULL;
 	g_autofree gchar *str = NULL;
 	g_auto(GStrv) strv = NULL;
+	GError *err;
 	guint i;
 	gchar *start, *end;
 	gchar *origin = NULL;
 
 	file = g_file_new_for_path (fname);
-	fistream = g_file_read (file, NULL, NULL);
+	fistream = g_file_read (file, NULL, &err);
+
+	if (!fistream) {
+		g_critical ("Unable to open file '%s' for reading: %s, skipping.", fname, err->message);
+		g_error_free (err);
+		return NULL;
+	}
+
 	mem_os = (GMemoryOutputStream*) g_memory_output_stream_new (NULL, 0, g_realloc, g_free);
 	zdecomp = g_zlib_decompressor_new (G_ZLIB_COMPRESSOR_FORMAT_GZIP);
 	conv_stream = g_converter_input_stream_new (G_INPUT_STREAM (fistream), G_CONVERTER (zdecomp));
@@ -158,12 +167,12 @@ as_extract_icon_cache_tarball (const gchar *asicons_target,
 }
 
 /**
- * as_data_pool_scan_apt:
+ * as_pool_scan_apt:
  *
  * Scan for additional metadata in 3rd-party directories and move it to the right place.
  */
 void
-as_data_pool_scan_apt (AsDataPool *dpool, gboolean force, GError **error)
+as_pool_scan_apt (AsPool *pool, gboolean force, GError **error)
 {
 	const gchar *apt_lists_dir = "/var/lib/apt/lists/";
 	const gchar *appstream_yml_target = "/var/lib/app-info/yaml";
@@ -240,7 +249,7 @@ as_data_pool_scan_apt (AsDataPool *dpool, gboolean force, GError **error)
 			const gchar *fname = (const gchar*) g_ptr_array_index (yml_files, i);
 			if (stat (fname, &sb) < 0)
 				continue;
-			if (sb.st_ctime > as_data_pool_get_cache_age (dpool)) {
+			if (sb.st_ctime > as_pool_get_cache_age (pool)) {
 				/* we need to update the cache */
 				data_changed = TRUE;
 				break;
@@ -275,7 +284,11 @@ as_data_pool_scan_apt (AsDataPool *dpool, gboolean force, GError **error)
 		fbasename = g_path_get_basename (fname);
 		dest_fname = g_build_filename (appstream_yml_target, fbasename, NULL);
 
-		if (!g_file_test (dest_fname, G_FILE_TEST_EXISTS)) {
+		if (!g_file_test (fname, G_FILE_TEST_EXISTS)) {
+			/* broken symlinks in the dest will have been removed earlier */
+			g_debug ("File %s is a broken symlink, skipping.", fname);
+			continue;
+		} else if (!g_file_test (dest_fname, G_FILE_TEST_EXISTS)) {
 			/* file not found, let's symlink */
 			if (symlink (fname, dest_fname) != 0) {
 				g_debug ("Unable to set symlink (%s -> %s): %s",
