@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014-2015 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2014-2016 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -41,7 +41,7 @@
 typedef struct
 {
 	AsProvidedKind	kind;
-	GHashTable	*items;
+	GPtrArray	*items;
 } AsProvidedPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsProvided, as_provided, G_TYPE_OBJECT)
@@ -168,7 +168,7 @@ as_provided_finalize (GObject *object)
 	AsProvided *prov = AS_PROVIDED (object);
 	AsProvidedPrivate *priv = GET_PRIVATE (prov);
 
-	g_hash_table_unref (priv->items);
+	g_ptr_array_unref (priv->items);
 
 	G_OBJECT_CLASS (as_provided_parent_class)->finalize (object);
 }
@@ -182,7 +182,7 @@ as_provided_init (AsProvided *prov)
 	AsProvidedPrivate *priv = GET_PRIVATE (prov);
 
 	priv->kind = AS_PROVIDED_KIND_UNKNOWN;
-	priv->items = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+	priv->items = g_ptr_array_new_with_free_func (g_free);
 }
 
 /**
@@ -238,7 +238,15 @@ gboolean
 as_provided_has_item (AsProvided *prov, const gchar *item)
 {
 	AsProvidedPrivate *priv = GET_PRIVATE (prov);
-	return g_hash_table_contains (priv->items, item);
+	guint i;
+
+	for (i = 0; i < priv->items->len; i++) {
+		const gchar *pitem = (const gchar*) g_ptr_array_index (priv->items, i);
+		if (g_strcmp0 (pitem, item) == 0)
+			return TRUE;
+	}
+
+	return FALSE;
 }
 
 /**
@@ -247,13 +255,13 @@ as_provided_has_item (AsProvided *prov, const gchar *item)
  *
  * Get an array of provided data.
  *
- * Returns: (transfer container): An utf-8 array of provided items, free with g_free()
+ * Returns: (transfer none) (element-type utf8): An string list of provided items.
  */
-gchar**
+GPtrArray*
 as_provided_get_items (AsProvided *prov)
 {
 	AsProvidedPrivate *priv = GET_PRIVATE (prov);
-	return (gchar**) g_hash_table_get_keys_as_array (priv->items, NULL);
+	return priv->items;
 }
 
 /**
@@ -266,7 +274,7 @@ void
 as_provided_add_item (AsProvided *prov, const gchar *item)
 {
 	AsProvidedPrivate *priv = GET_PRIVATE (prov);
-	g_hash_table_add (priv->items, g_strdup (item));
+	g_ptr_array_add (priv->items, g_strdup (item));
 }
 
 /**
