@@ -23,7 +23,7 @@
  * @short_description: Validator and report-generator about AppStream XML metadata
  * @include: appstream.h
  *
- * This object is able to validate AppStream XML metadata (distro and upstream)
+ * This object is able to validate AppStream XML metadata (collection and metainfo)
  * and to generate a report about issues found with it.
  *
  * See also: #AsMetadata
@@ -301,14 +301,14 @@ as_validator_check_nolocalized (AsValidator *validator, xmlNode* node, const gch
  * as_validator_check_description_tag:
  **/
 static void
-as_validator_check_description_tag (AsValidator *validator, xmlNode* node, AsComponent *cpt, AsParserMode mode)
+as_validator_check_description_tag (AsValidator *validator, xmlNode* node, AsComponent *cpt, AsFormatStyle mode)
 {
 	xmlNode *iter;
 	gchar *node_content;
 	gchar *node_name;
 	gboolean first_paragraph = TRUE;
 
-	if (mode == AS_PARSER_MODE_UPSTREAM) {
+	if (mode == AS_FORMAT_STYLE_METAINFO) {
 		as_validator_check_nolocalized (validator,
 						node,
 						(const gchar*) node->name,
@@ -332,12 +332,12 @@ as_validator_check_description_tag (AsValidator *validator, xmlNode* node, AsCom
 		}
 
 		if (g_strcmp0 (node_name, "p") == 0) {
-			if (mode == AS_PARSER_MODE_DISTRO) {
+			if (mode == AS_FORMAT_STYLE_COLLECTION) {
 				as_validator_check_nolocalized (validator,
 								iter,
 								"description/p",
 								cpt,
-								"The '%s' tag should not be localized in distro metadata. Localize the whole 'description' tag instead.");
+								"The '%s' tag should not be localized in collection metadata. Localize the whole 'description' tag instead.");
 			}
 			if ((first_paragraph) && (strlen (node_content) < 100)) {
 				as_validator_add_issue (validator, iter,
@@ -348,21 +348,21 @@ as_validator_check_description_tag (AsValidator *validator, xmlNode* node, AsCom
 			}
 			first_paragraph = FALSE;
 		} else if (g_strcmp0 (node_name, "ul") == 0) {
-			if (mode == AS_PARSER_MODE_DISTRO) {
+			if (mode == AS_FORMAT_STYLE_COLLECTION) {
 				as_validator_check_nolocalized (validator,
 								iter,
 								"description/ul",
 								cpt,
-								"The '%s' tag should not be localized in distro metadata. Localize the whole 'description' tag instead.");
+								"The '%s' tag should not be localized in collection metadata. Localize the whole 'description' tag instead.");
 			}
 			as_validator_check_children_quick (validator, iter, "li", cpt);
 		} else if (g_strcmp0 (node_name, "ol") == 0) {
-			if (mode == AS_PARSER_MODE_DISTRO) {
+			if (mode == AS_FORMAT_STYLE_COLLECTION) {
 				as_validator_check_nolocalized (validator,
 								iter,
 								"description/ol",
 								cpt,
-								"The '%s' tag should not be localized in distro metadata. Localize the whole 'description' tag instead.");
+								"The '%s' tag should not be localized in collection metadata. Localize the whole 'description' tag instead.");
 			}
 			as_validator_check_children_quick (validator, iter, "li", cpt);
 		} else {
@@ -408,69 +408,28 @@ as_validator_check_appear_once (AsValidator *validator, xmlNode *node, GHashTabl
 }
 
 /**
- * as_validator_validate_component_node:
- **/
-static AsComponent*
-as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xmlNode *root)
+ * as_validator_validate_component_id:
+ *
+ * Validate the component-ID.
+ */
+static void
+as_validator_validate_component_id (AsValidator *validator, xmlNode *idnode, AsComponent *cpt)
 {
-	xmlNode *iter;
-	AsComponent *cpt;
 	guint i;
-	g_autofree gchar *cpttype = NULL;
-	g_autoptr(GHashTable) found_tags = NULL;
 	g_auto(GStrv) cid_parts = NULL;
-	const gchar *summary;
-	const gchar *cid;
-	AsParserMode mode;
-	gboolean has_metadata_license = FALSE;
+	g_autofree gchar *cid = (gchar*) xmlNodeGetContent (idnode);
 
-	found_tags = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-	mode = as_xmldata_get_parser_mode (xdt);
-
-	/* validate the resulting AsComponent for sanity */
-	cpt = as_component_new ();
-	as_xmldata_parse_component_node (xdt, root, cpt, NULL);
-	as_validator_set_current_cpt (validator, cpt);
-
-	/* check if component type is valid */
-	cid = as_component_get_id (cpt);
-	cpttype = (gchar*) xmlGetProp (root, (xmlChar*) "type");
-	if (cpttype != NULL) {
-		if (as_component_kind_from_string (cpttype) == AS_COMPONENT_KIND_UNKNOWN) {
-			as_validator_add_issue (validator, root,
-						AS_ISSUE_IMPORTANCE_ERROR,
-						AS_ISSUE_KIND_VALUE_WRONG,
-						"Invalid component type found: %s",
-						cpttype);
-		}
-	}
-
-	if ((as_component_get_priority (cpt) != 0) && (mode == AS_PARSER_MODE_UPSTREAM)) {
-		as_validator_add_issue (validator, root,
-					AS_ISSUE_IMPORTANCE_ERROR,
-					AS_ISSUE_KIND_VALUE_WRONG,
-					"The component has a priority value set. This is not allowed in metainfo files.");
-	}
-
-	if ((as_component_get_merge_kind (cpt) != AS_MERGE_KIND_NONE) && (mode == AS_PARSER_MODE_UPSTREAM)) {
-		as_validator_add_issue (validator, root,
-					AS_ISSUE_IMPORTANCE_ERROR,
-					AS_ISSUE_KIND_VALUE_WRONG,
-					"The component has a 'merge' method defined. This is not allowed in metainfo files.");
-	}
-
-	/* validate the AppStream ID */
 	cid_parts = g_strsplit (cid, ".", 3);
 	if (g_strv_length (cid_parts) != 3) {
 		if (as_component_get_kind (cpt) == AS_COMPONENT_KIND_DESKTOP_APP) {
 			/* since the ID and .desktop-file-id are tied together, we can't make this an error for desktop apps */
-			as_validator_add_issue (validator, NULL,
+			as_validator_add_issue (validator, idnode,
 					AS_ISSUE_IMPORTANCE_WARNING,
 					AS_ISSUE_KIND_VALUE_WRONG,
-					"The component ID is not a reverse domain-name. Please update the ID and that of the accompanying .desktop file to follow the latest version of the specifications and avoid future issues.");
+					"The component ID is not a reverse domain-name. Please update the ID and that of the accompanying .desktop file to follow the latest version of the Desktop-Entry and AppStream specifications and avoid future issues.");
 		} else {
-			/* anything which isn't a .desktop app should follow the schema though */
-			as_validator_add_issue (validator, NULL,
+			/* anything which isn't a .desktop app must follow the schema though */
+			as_validator_add_issue (validator, idnode,
 					AS_ISSUE_IMPORTANCE_ERROR,
 					AS_ISSUE_KIND_VALUE_WRONG,
 					"The component ID is no reverse domain-name.");
@@ -479,7 +438,7 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 		/* some people just add random dots to their ID - check if we have an actual known TLD as first part, to be more certain that this is a reverse domain name
 		 * (this issue happens quite often with old .desktop files) */
 		if (!as_utils_is_tld (cid_parts[0])) {
-			as_validator_add_issue (validator, NULL,
+			as_validator_add_issue (validator, idnode,
 						AS_ISSUE_IMPORTANCE_INFO,
 						AS_ISSUE_KIND_VALUE_WRONG,
 						"The component ID might not follow the reverse domain-name schema (we do not know about the TLD '%s').", cid_parts[0]);
@@ -495,14 +454,76 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 		    (cid[i] != '_')) {
 			g_autofree gchar *c = NULL;
 			c = g_utf8_substring (cid, i, i + 1);
-			as_validator_add_issue (validator, NULL,
+			as_validator_add_issue (validator, idnode,
 					AS_ISSUE_IMPORTANCE_ERROR,
 					AS_ISSUE_KIND_VALUE_WRONG,
-					"The component ID contains an invalid character: '%s'", c);
+					"The component ID [%s] contains an invalid character: '%s'", cid, c);
 		}
 	}
 
+	/* project-group specific constraints on the ID */
+	if (g_strcmp0 (as_component_get_project_group (cpt), "KDE") == 0) {
+		if (!g_str_has_prefix (cid, "org.kde."))
+			as_validator_add_issue (validator, idnode,
+						AS_ISSUE_IMPORTANCE_ERROR,
+						AS_ISSUE_KIND_VALUE_WRONG,
+						"The component is part of the KDE project, but its id does not start with KDEs reverse-DNS name (\"org.kde\").");
+	} else if (g_strcmp0 (as_component_get_project_group (cpt), "GNOME") == 0) {
+		if (!g_str_has_prefix (cid, "org.gnome."))
+			as_validator_add_issue (validator, idnode,
+						AS_ISSUE_IMPORTANCE_ERROR,
+						AS_ISSUE_KIND_VALUE_WRONG,
+						"The component is part of the GNOME project, but its id does not start with GNOMEs reverse-DNS name (\"org.gnome\").");
+	}
+}
 
+/**
+ * as_validator_validate_component_node:
+ **/
+static AsComponent*
+as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xmlNode *root)
+{
+	xmlNode *iter;
+	AsComponent *cpt;
+	g_autofree gchar *cpttype = NULL;
+	g_autoptr(GHashTable) found_tags = NULL;
+
+	AsFormatStyle mode;
+	gboolean has_metadata_license = FALSE;
+
+	found_tags = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+	mode = as_xmldata_get_format_style (xdt);
+
+	/* validate the resulting AsComponent for sanity */
+	cpt = as_component_new ();
+	as_xmldata_parse_component_node (xdt, root, cpt, NULL);
+	as_validator_set_current_cpt (validator, cpt);
+
+	/* check if component type is valid */
+	cpttype = (gchar*) xmlGetProp (root, (xmlChar*) "type");
+	if (cpttype != NULL) {
+		if (as_component_kind_from_string (cpttype) == AS_COMPONENT_KIND_UNKNOWN) {
+			as_validator_add_issue (validator, root,
+						AS_ISSUE_IMPORTANCE_ERROR,
+						AS_ISSUE_KIND_VALUE_WRONG,
+						"Invalid component type found: %s",
+						cpttype);
+		}
+	}
+
+	if ((as_component_get_priority (cpt) != 0) && (mode == AS_FORMAT_STYLE_METAINFO)) {
+		as_validator_add_issue (validator, root,
+					AS_ISSUE_IMPORTANCE_ERROR,
+					AS_ISSUE_KIND_VALUE_WRONG,
+					"The component has a priority value set. This is not allowed in metainfo files.");
+	}
+
+	if ((as_component_get_merge_kind (cpt) != AS_MERGE_KIND_NONE) && (mode == AS_FORMAT_STYLE_METAINFO)) {
+		as_validator_add_issue (validator, root,
+					AS_ISSUE_IMPORTANCE_ERROR,
+					AS_ISSUE_KIND_VALUE_WRONG,
+					"The component has a 'merge' method defined. This is not allowed in metainfo files.");
+	}
 
 	for (iter = root->children; iter != NULL; iter = iter->next) {
 		const gchar *node_name;
@@ -533,12 +554,15 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 								"Component id belongs to a desktop-application, but does not resemble the .desktop file name: \"%s\"",
 								node_content);
 			}
+
+			/* validate the AppStream ID */
+			as_validator_validate_component_id (validator, iter, cpt);
 		} else if (g_strcmp0 (node_name, "metadata_license") == 0) {
 			has_metadata_license = TRUE;
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 
 			/* the license must allow easy mixing of metadata in metainfo files */
-			if (mode == AS_PARSER_MODE_UPSTREAM) {
+			if (mode == AS_FORMAT_STYLE_METAINFO) {
 				if (!as_license_is_metadata_license (node_content)) {
 					as_validator_add_issue (validator, iter,
 								AS_ISSUE_IMPORTANCE_WARNING,
@@ -558,8 +582,31 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 		} else if (g_strcmp0 (node_name, "name") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
+			if (g_str_has_suffix (node_content, ".")) {
+				as_validator_add_issue (validator, iter,
+							AS_ISSUE_IMPORTANCE_INFO,
+							AS_ISSUE_KIND_VALUE_ISSUE,
+							"The component name should not end with a \".\" [%s]",
+							node_content);
+			}
+
 		} else if (g_strcmp0 (node_name, "summary") == 0) {
+			const gchar *summary = node_content;
+
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
+			if (g_str_has_suffix (summary, "."))
+				as_validator_add_issue (validator, iter,
+							AS_ISSUE_IMPORTANCE_INFO,
+							AS_ISSUE_KIND_VALUE_ISSUE,
+							"The component summary should not end with a \".\" [%s]",
+							summary);
+
+			if ((summary != NULL) && ((strstr (summary, "\n") != NULL) || (strstr (summary, "\t") != NULL))) {
+				as_validator_add_issue (validator, iter,
+							AS_ISSUE_IMPORTANCE_ERROR,
+							AS_ISSUE_KIND_VALUE_WRONG,
+							"The summary tag must not contain tabs or linebreaks.");
+			}
 		} else if (g_strcmp0 (node_name, "description") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 			as_validator_check_description_tag (validator, iter, cpt, mode);
@@ -605,13 +652,18 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 		} else if (g_strcmp0 (node_name, "developer_name") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 		} else if (g_strcmp0 (node_name, "compulsory_for_desktop") == 0) {
-			as_validator_check_appear_once (validator, iter, found_tags, cpt);
+			if (!as_utils_is_desktop_environment (node_content)) {
+				as_validator_add_issue (validator, iter,
+							AS_ISSUE_IMPORTANCE_ERROR,
+							AS_ISSUE_KIND_VALUE_WRONG,
+							"Unknown desktop-id '%s'.", node_content);
+			}
 		} else if (g_strcmp0 (node_name, "releases") == 0) {
 			as_validator_check_children_quick (validator, iter, "release", cpt);
-		} else if ((g_strcmp0 (node_name, "languages") == 0) && (mode == AS_PARSER_MODE_DISTRO)) {
+		} else if ((g_strcmp0 (node_name, "languages") == 0) && (mode == AS_FORMAT_STYLE_COLLECTION)) {
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 			as_validator_check_children_quick (validator, iter, "lang", cpt);
-		} else if ((g_strcmp0 (node_name, "translation") == 0) && (mode == AS_PARSER_MODE_UPSTREAM)) {
+		} else if ((g_strcmp0 (node_name, "translation") == 0) && (mode == AS_FORMAT_STYLE_METAINFO)) {
 			g_autofree gchar *prop = NULL;
 			AsTranslationKind trkind;
 			prop = as_validator_check_type_property (validator, cpt, iter);
@@ -626,33 +678,37 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 		} else if (g_strcmp0 (node_name, "bundle") == 0) {
 			g_autofree gchar *prop = NULL;
 			prop = as_validator_check_type_property (validator, cpt, iter);
-			if ((g_strcmp0 (prop, "limba") != 0) && (g_strcmp0 (prop, "flatpak") != 0)) {
+			if (as_bundle_kind_from_string (prop) == AS_BUNDLE_KIND_UNKNOWN) {
 				as_validator_add_issue (validator, iter,
 							AS_ISSUE_IMPORTANCE_ERROR,
 							AS_ISSUE_KIND_VALUE_WRONG,
 							"Unknown type '%s' for <bundle/> tag.", prop);
 			}
 		} else if (g_strcmp0 (node_name, "update_contact") == 0) {
-			if (mode == AS_PARSER_MODE_DISTRO) {
+			if (mode == AS_FORMAT_STYLE_COLLECTION) {
 				as_validator_add_issue (validator, iter,
 							AS_ISSUE_IMPORTANCE_WARNING,
 							AS_ISSUE_KIND_TAG_NOT_ALLOWED,
-							"The 'update_contact' tag should not be included in distro AppStream XML.");
+							"The 'update_contact' tag should not be included in collection AppStream XML.");
 			} else {
 				as_validator_check_appear_once (validator, iter, found_tags, cpt);
 			}
-		} else if (g_strcmp0 (node_name, "metadata") == 0) {
+		} else if ((g_strcmp0 (node_name, "metadata") == 0) || (g_strcmp0 (node_name, "kudos") == 0)) {
+			/* these tags are GNOME / Fedora specific extensions and are therefore quite common. They shouldn't make the validation fail,
+			 * especially if we might standardize at leat the <kudos/> tag one day, but we should still complain about those tags to make
+			 * it obvious that they are not supported by all implementations */
 			as_validator_add_issue (validator, iter,
-						AS_ISSUE_IMPORTANCE_PEDANTIC,
+						AS_ISSUE_IMPORTANCE_INFO,
 						AS_ISSUE_KIND_TAG_UNKNOWN,
-						"Found custom metadata in <metadata/> tag. Use of this tag is common, but should be avoided if possible.");
+						"Found invalid tag: '%s'. This tag is a GNOME extensions to AppStream and is not supported by all implementations.",
+						node_name);
 			tag_valid = FALSE;
 		} else if (!g_str_has_prefix (node_name, "x-")) {
 			as_validator_add_issue (validator, iter,
 						AS_ISSUE_IMPORTANCE_WARNING,
 						AS_ISSUE_KIND_TAG_UNKNOWN,
 						"Found invalid tag: '%s'. Non-standard tags must be prefixed with \"x-\".",
-				node_name);
+						node_name);
 			tag_valid = FALSE;
 		}
 
@@ -666,22 +722,13 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 	}
 
 	/* emit an error if we are missing the metadata license in metainfo files */
-	if ((!has_metadata_license) && (mode == AS_PARSER_MODE_UPSTREAM)) {
+	if ((!has_metadata_license) && (mode == AS_FORMAT_STYLE_METAINFO)) {
 		as_validator_add_issue (validator, NULL,
 					AS_ISSUE_IMPORTANCE_ERROR,
 					AS_ISSUE_KIND_TAG_MISSING,
 					"The essential tag 'metadata_license' is missing.");
 	}
 
-	/* check if the summary is sane */
-	summary = as_component_get_summary (cpt);
-	if ((summary != NULL) && ((strstr (summary, "\n") != NULL) || (strstr (summary, "\t") != NULL))) {
-		as_validator_add_issue (validator, NULL,
-					AS_ISSUE_IMPORTANCE_ERROR,
-					AS_ISSUE_KIND_VALUE_WRONG,
-					"The summary tag must not contain tabs or linebreaks.");
-	}
-
 	/* check if we have a description */
 	if (as_str_empty (as_component_get_description (cpt))) {
 		AsComponentKind cpt_kind;
@@ -720,13 +767,23 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 					"Type 'font' component, but no font information was provided via a provides/font tag.");
 	}
 
+	/* validate driver specific stuff */
+	if (as_component_get_kind (cpt) == AS_COMPONENT_KIND_DRIVER) {
+		if (as_component_get_provided_for_kind (cpt, AS_PROVIDED_KIND_MODALIAS) == NULL)
+			as_validator_add_issue (validator, NULL,
+					AS_ISSUE_IMPORTANCE_INFO,
+					AS_ISSUE_KIND_TAG_MISSING,
+					"Type 'driver' component, but no modalias information was provided via a provides/modalias tag.");
+	}
+
 	/* validate addon specific stuff */
 	if (as_component_get_extends (cpt)->len > 0) {
-		if (as_component_get_kind (cpt) != AS_COMPONENT_KIND_ADDON)
+		AsComponentKind kind = as_component_get_kind (cpt);
+		if ((kind != AS_COMPONENT_KIND_ADDON) && (kind != AS_COMPONENT_KIND_LOCALIZATION))
 			as_validator_add_issue (validator, NULL,
 						AS_ISSUE_IMPORTANCE_ERROR,
 						AS_ISSUE_KIND_TAG_NOT_ALLOWED,
-						"An 'extends' tag is specified, but the component is not an addon.");
+						"An 'extends' tag is specified, but the component is not of type 'addon' or 'localization'.");
 	} else {
 		if (as_component_get_kind (cpt) == AS_COMPONENT_KIND_ADDON)
 			as_validator_add_issue (validator, NULL,
@@ -735,6 +792,22 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 						"The component is an addon, but no 'extends' tag was specified.");
 	}
 
+	/* validate l10n specific stuff */
+	if (as_component_get_kind (cpt) == AS_COMPONENT_KIND_LOCALIZATION) {
+		if (as_component_get_extends (cpt)->len == 0) {
+			as_validator_add_issue (validator, NULL,
+						AS_ISSUE_IMPORTANCE_WARNING,
+						AS_ISSUE_KIND_TAG_MISSING,
+						"This 'localization' component is missing an An 'extends' tag, to specify the components it adds localization to.");
+		}
+		if (g_hash_table_size (as_component_get_languages_table (cpt)) == 0) {
+			as_validator_add_issue (validator, NULL,
+						AS_ISSUE_IMPORTANCE_ERROR,
+						AS_ISSUE_KIND_TAG_MISSING,
+						"This 'localization' component does not define any languages this localization is for.");
+		}
+	}
+
 	as_validator_clear_current_cpt (validator);
 	return cpt;
 }
@@ -855,7 +928,12 @@ as_validator_validate_data (AsValidator *validator, const gchar *metadata)
 
 	/* load the XML data */
 	xdt = as_xmldata_new ();
-	as_xmldata_initialize (xdt, "C", NULL, NULL, NULL, 0);
+	as_xmldata_initialize (xdt, AS_CURRENT_FORMAT_VERSION,
+			       "C",
+				NULL,
+				NULL,
+				NULL,
+				0);
 
 	doc = as_validator_open_xml_document (validator, xdt, metadata);
 	if (doc == NULL)
@@ -864,7 +942,7 @@ as_validator_validate_data (AsValidator *validator, const gchar *metadata)
 
 	ret = TRUE;
 	if (g_strcmp0 ((gchar*) root->name, "component") == 0) {
-		as_xmldata_set_parser_mode (xdt, AS_PARSER_MODE_UPSTREAM);
+		as_xmldata_set_format_style (xdt, AS_FORMAT_STYLE_METAINFO);
 		cpt = as_validator_validate_component_node (validator, xdt, root);
 		if (cpt != NULL)
 			g_object_unref (cpt);
@@ -872,7 +950,7 @@ as_validator_validate_data (AsValidator *validator, const gchar *metadata)
 		xmlNode *iter;
 		const gchar *node_name;
 
-		as_xmldata_set_parser_mode (xdt, AS_PARSER_MODE_DISTRO);
+		as_xmldata_set_format_style (xdt, AS_FORMAT_STYLE_COLLECTION);
 		for (iter = root->children; iter != NULL; iter = iter->next) {
 			/* discard spaces */
 			if (iter->type != XML_ELEMENT_NODE)
@@ -998,7 +1076,7 @@ as_validator_analyze_component_metainfo_relation_cb (const gchar *fname, AsCompo
 				/* we successfully opened the .desktop file, now perform some checks */
 
 				/* name */
-				if ((g_strcmp0 (as_component_get_name (cpt), "") == 0) &&
+				if (as_str_empty (as_component_get_name (cpt)) &&
 				    (!g_key_file_has_key (dfile, G_KEY_FILE_DESKTOP_GROUP,
 								 G_KEY_FILE_DESKTOP_KEY_NAME, NULL))) {
 					/* we don't have a summary, and there is also none in the .desktop file - this is bad. */
@@ -1009,7 +1087,7 @@ as_validator_analyze_component_metainfo_relation_cb (const gchar *fname, AsCompo
 				}
 
 				/* summary */
-				if ((g_strcmp0 (as_component_get_summary (cpt), "") == 0) &&
+				if (as_str_empty (as_component_get_summary (cpt)) &&
 				    (!g_key_file_has_key (dfile, G_KEY_FILE_DESKTOP_GROUP,
 								 G_KEY_FILE_DESKTOP_KEY_COMMENT, NULL))) {
 					/* we don't have a summary, and there is also none in the .desktop file - this is bad. */
@@ -1110,8 +1188,13 @@ as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 
 	/* set up XML parser */
 	xdt = as_xmldata_new ();
-	as_xmldata_initialize (xdt, "C", NULL, NULL, NULL, 0);
-	as_xmldata_set_parser_mode (xdt, AS_PARSER_MODE_UPSTREAM);
+	as_xmldata_initialize (xdt, AS_CURRENT_FORMAT_VERSION,
+			       "C",
+				NULL,
+				NULL,
+				NULL,
+				0);
+	as_xmldata_set_format_style (xdt, AS_FORMAT_STYLE_METAINFO);
 
 	/* validate all metainfo files */
 	mfiles = as_utils_find_files_matching (metainfo_dir, "*.xml", FALSE, NULL);
